@@ -4,17 +4,26 @@ use 5.006;
 use strict;
 use warnings;
 
+our @checksum_algos = qw(md5 sha1);
+
+use File::Find;
+
+=head1 WARNING
+
+This is experimental software for the moment and under active development. I
+hope to have a beta version available soon.
+
 =head1 NAME
 
 Archive::BagIt - An interface to make and verify bags according to the BagIt standard
 
 =head1 VERSION
 
-Version 0.02
+Version 0.03
 
 =cut
 
-our $VERSION = '0.02';
+our $VERSION = '0.03';
 
 
 =head1 SYNOPSIS
@@ -49,6 +58,7 @@ and I will endeavour to maintain compatibility with it.
 sub new {
   my ($class,$bag_path) = @_;
   my $self = {};
+  $bag_path=~s!/$!!;
   $self->{'bag_path'} = $bag_path || "";
   bless $self, $class;
   return $self;
@@ -56,14 +66,23 @@ sub new {
 
 =head2 make_bag
    A constructor that will make and return a bag from a directory
+
+   If a data directory exists, assume it is already a bag (no checking for invalid files in root)
 =cut
 
 sub make_bag {
   my ($class, $bag_dir) = @_;
+  unless ( -d $bag_dir) { die ( "source bag directory doesn't exist"); }
+  unless ( -d $bag_dir."/data") {
+    rename ($bag_dir, $bag_dir.".tmp");
+    mkdir  ($bag_dir);
+    rename ($bag_dir.".tmp", $bag_dir."/data");
+  }
   my $self=$class->new($bag_dir);
   $self->_write_bagit($bag_dir);
   $self->_write_baginfo($bag_dir);
   $self->_manifest_md5($bag_dir);
+  $self->_tagmanifest_md5($bag_dir);
   return $self;
 }
 
@@ -91,6 +110,7 @@ sub _write_baginfo {
 }
 
 sub _manifest_crc32 {
+    require String::CRC32;
     my($self,$bagit) = @_;
     my $manifest_file = "$bagit/manifest-crc32.txt";
     my $data_dir = "$bagit/data";
@@ -116,8 +136,7 @@ sub _manifest_crc32 {
 
 
 sub _manifest_md5 {
-    use File::Find;
-    use Digest::MD5 qw/md5_hex/;
+    use Digest::MD5;
     my($self, $bagit) = @_;
     my $manifest_file = "$bagit/manifest-md5.txt";
     my $data_dir = "$bagit/data";
@@ -130,7 +149,7 @@ sub _manifest_md5 {
             my $file = $File::Find::name;
             if (-f $_) {
                 open(DATA, "<$_") or die("Cannot read $_: $!");
-                my $digest = md5_hex(join("", <DATA>));
+                my $digest = Digest::MD5->new->addfile(*DATA)->hexdigest;
                 close(DATA);
                 my $filename = substr($file, length($bagit) + 1);
                 print($md5_fh "$digest  $filename\n");
@@ -141,6 +160,36 @@ sub _manifest_md5 {
     close(MD5);
 }
 
+sub _tagmanifest_md5 {
+  my ($self, $bagit) = @_;
+
+  use Digest::MD5;
+
+  my $tagmanifest_file= "$bagit/tagmanifest-md5.txt";
+ 
+  open (MD5, ">$tagmanifest_file") or die ("Cannot create tagmanifest-md5.txt: $! \n");
+
+  my $md5_fh = *MD5;
+  find (
+    sub {
+      my $file = $File::Find::name;
+      if (-f $_ && $_=~m/^tagmanifest-.*\.txt/) {
+        open(DATA, "<$_") or die("Cannot read $_: $!");
+        my $digest = Digest::MD5->new->addfile(*DATA)->hexdigest;
+        close(DATA);
+        my $filename = substr($file, length($bagit) + 1);
+        print($md5_fh "$digest  $filename\n");
+ 
+      }
+      elsif($_=~m/\/data$/) {
+        $File::Find::prune=1;
+      }
+
+  }, $bagit);
+
+  close(MD5);
+}
+
 =head2 verify_bag
 
 An interface to verify a bag
@@ -149,7 +198,15 @@ An interface to verify a bag
 
 sub verify_bag {
     my ($self,$bagit) = @_;
-    $self->{'bag_path'} = $bagit;
+    if($bagit) {
+      $self->{'bag_path'} = $bagit;
+    }
+    elsif ($self->{'bag_path'}) {
+      $bagit = $self->{'bag_path'};
+    }
+    else {
+      die ("no bag_path defined");
+    }
     my $manifest_file = "$bagit/manifest-md5.txt";
     my $payload_dir   = "$bagit/data";
     my %manifest      = ();
@@ -204,6 +261,7 @@ sub get_checksum {
 
 =head2 version
 
+   Returns the bagit version according to the bagit.txt file.
 =cut
 
 sub version {
@@ -218,20 +276,74 @@ sub version {
     return $1 || 0;
 }
 
+=head2 payload_files
 
+  Returns an array with all of the payload files (those files that are below the data directory)
+
+=cut
+
+sub payload_files {
+  my($self) = @_;
+  my @payload = $self->_payload_files();
+  return @payload; 
+}
+
+sub _payload_files{
+  my($self) = @_;
+
+  my $payload_dir = join( "/", $self->{"bag_path"}, "data");
+  
+  my @payload=();
+  File::Find::find( sub{ 
+    push(@payload,$File::Find::name); 
+    #print "name: ".$File::Find::name."\n"; 
+  }, $payload_dir);
+  
+  return @payload;
+
+}
+
+=head2 manifest_files
+
+  return an array with the list of manifest files that exist in the bag
+
+=cut
+
+sub manifest_files {
+  my($self) = @_;
+  my @manifest_files;
+  foreach my $algo (@checksum_algos) {
+    my $manifest_file = $self->{"bag_path"}."/manifest-$algo.txt";
+    if (-f $manifest_file) {
+      push @manifest_files, $manifest_file;
+    }
+  }
+  return @manifest_files;
+}
+
+=head2 tagmanifest_files
+  
+  return an array with the list of tagmanifest files
+
+=cut
+
+sub tagmanifest_files {
+  my ($self) = @_;
+  my @tagmanifest_files;
+  foreach my $algo (@checksum_algos) {
+    my $tagmanifest_file = $self->{"bag_path"}."/tagmanifest-$algo.txt";
+    if (-f $tagmanifest_file) {
+      push @tagmanifest_files, $tagmanifest_file;
+    }
+  }
+  return @tagmanifest_files;
+  
+}
 =head1 AUTHOR
 
-=over 4
+Robert Schmidt, E<lt>rjeschmi at gmail.comE<gt>
+William Wueppelmann, E<lt>william at c7a.caE<gt>
 
-=item *
-
-Robert Schmidt, C<< <rjeschmi at gmail.com> >> 
-
-=item *
-
-William Wueppelmann, C<< <william at c7a.ca> >>
-
-=back
 
 =head1 BUGS
 
@@ -272,15 +384,15 @@ L<http://search.cpan.org/dist/Archive-BagIt/>
 =back
 
 
-=head1 ACKNOWLEDGEMENTS
+=head1 COPYRIGHT
+
+Copyright (c) 2012, the above named author(s).
 
 
-=head1 LICENSE AND COPYRIGHT
-
-Copyright 2012 Robert Schmidt and William Wueppelmann
-
-This program is released under the following license: cc0
+=head1 LICENSE
 
+This library is free software; you can redistribute it and/or modify
+it under the same terms as Perl itself.
 
 =cut
 
